//---------------------------------------------------------------------------
#pragma hdrstop
#include "grapEllipse.h"
#include "ASMVars.h"
#include <math.h>
#include <algorithm>
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// Graphic primitives: ellipse
// Author: Sinia Petri
//---------------------------------------------------------------------------
__fastcall TGrapEllipse::TGrapEllipse(int imgHeight)
           : TGrapBasic(imgHeight)
{
structPtChunk *cItem;
for (int i= 0; i < 4; i++)  // four quadrants
    {
    chunkList[i] = new TObjectList(false);
    cItem = new structPtChunk;
    cItem->Count = 0;
    chunkList[i]->Add((TObject*)cItem);
    chunkItem[i] = cItem;
    }
}
//---------------------------------------------------------------------------
// destructor
__fastcall TGrapEllipse::~TGrapEllipse(void)
{
structPtChunk *cItem;
TObjectList *cList;
int i;
for (i = 0; i < 4; i++) // four quadrants
    {
    cList = chunkList[i];
    while (chunkList[i]->Count > 0)
       {
       cItem = reinterpret_cast <structPtChunk*>(cList->Items[0]);
       delete cItem;
       chunkList[i]->Delete(0);
       }
    delete cList;
    }
}
//---------------------------------------------------------------------------
UnicodeString __fastcall TGrapEllipse::ClassName(void)
{
return "TGrapEllipse";
}
//---------------------------------------------------------------------------
// public method - implementation
//---------------------------------------------------------------------------
bool __fastcall TGrapEllipse::DrawBound(TRect *boundRect)
{
if (boundRect->Width() == 0 || boundRect->Height() == 0)
   return false;
int rx = (boundRect->Width()>>1);
int ry = (boundRect->Height()>>1);
int xc = rx + boundRect->Left;
int yc = ry + boundRect->Top;
DrawCoords(xc, yc, rx,ry);
return true;
}
//---------------------------------------------------------------------------
// public method - implementation
//---------------------------------------------------------------------------
void __fastcall TGrapEllipse::DrawCoords(int xc, int yc, int rx, int ry)
{
// midpoint algorithm
Reset();
int x = 0, y = ry;
int p =(ry*ry)-(rx*rx*ry)+((rx*rx)/4);
while ((2*x*ry*ry)<(2*y*rx*rx))
   {
   savePoint(chunkList[0], chunkItem[0], xc+x,yc-y); // 1. quadrant
   savePoint(chunkList[1], chunkItem[1], xc-x,yc-y); // 2. quadant
   savePoint(chunkList[2], chunkItem[2], xc-x,yc+y); // 3. quadrant
   savePoint(chunkList[3], chunkItem[3], xc+x,yc+y); // 4. qaudrant
   if ( p < 0)
      {
      x = x + 1;
      p = p +(2*ry*ry*x)+(ry*ry);
      }
   else
      {
      x = x + 1;
      y = y - 1;
      p = p + (2*ry*ry*x+ry*ry)-(2*rx*rx*y);
      }
   }
   p = ((float)x+0.5)*((float)x+0.5)*ry*ry+(y-1)*(y-1)*rx*rx-rx*rx*ry*ry;
while( y >= 0)
   {
   savePoint(chunkList[0], chunkItem[0], xc+x,yc-y); // 1. quadrant
   savePoint(chunkList[1], chunkItem[1], xc-x,yc-y); // 2. quadant
   savePoint(chunkList[2], chunkItem[2], xc-x,yc+y); // 3. quadrant
   savePoint(chunkList[3], chunkItem[3], xc+x,yc+y); // 4. qaudrant
   if (p > 0)
      {
      y = y-1;
      p = p-(2*rx*rx*y)+(rx*rx);
      }
   else
      {
      y = y-1;
      x = x+1;
      p = p+(2*ry*ry*x)-(2*rx*rx*y)-(rx*rx);
      }
  }
}
//---------------------------------------------------------------------------
// public method - implementation
//---------------------------------------------------------------------------
void __fastcall TGrapEllipse::GetBitmapPoints(int lineStep)
{
int i, j, k, ptCount = 0; //, step = 0;
int chunkSize = 0;
for (i = 0; i < 4; i++)
    chunkSize += getOverallChunkSize(chunkList[i]);
if (PointBmp)
   delete[] PointBmp;
PointBmp = new TPoint[chunkSize];
// collect chunks in one array
TObjectList *cList;
structPtChunk *cItem;
for (i = 0; i < 4; i++)
//for (i = 0; i < 3; i++)
    {
    cList = chunkList[i];
    if (i == 0 || i == 2)
       {
        for (j = cList->Count - 1; j >= 0; j--)
            {
            cItem = reinterpret_cast <structPtChunk*>(cList->Items[j]);
            for (k = cItem->Count - 1; k >= 0; k--)
               {
               //if (++step < lineStep)
               if (++_Step < lineStep)
                  continue;
               PointBmp[ptCount].x  = cItem->Point[k].x;
               PointBmp[ptCount++].y = _imgHeightM1 - cItem->Point[k].y;
               //step = 0;
               _Step = 0;
               }
            }
        }
    else
        {
        for (j = 0; j < cList->Count; j++)
            {
            cItem = reinterpret_cast <structPtChunk*>(cList->Items[j]);
            for (k = 0; k < cItem->Count; k++)
               {
               //if (++step < lineStep)
               if (++_Step < lineStep)
                  continue;
               PointBmp[ptCount].x  = cItem->Point[k].x;
               PointBmp[ptCount++].y = _imgHeightM1 - cItem->Point[k].y;
               //step = 0;
               _Step = 0;
			   }
			}
		}
	}
CountBmp = ptCount;
}
//---------------------------------------------------------------------------
void __fastcall TGrapEllipse::GetScreenPoints(int lineStep)
{
int i, j, k, ptCount = 0; //, step = 0;
int chunkSize = 0;
for (i = 0; i < 4; i++)
    chunkSize += getOverallChunkSize(chunkList[i]);
if (PointScreen)
   delete[] PointScreen;
PointScreen = new TPoint[chunkSize];
// collect chunks in one array
TObjectList *cList;
structPtChunk *cItem;
for (i = 0; i < 4; i++)
	{
	cList = chunkList[i];
	if (i == 0 || i == 2)
	   {
		for (j = cList->Count - 1; j >= 0; j--)
			{
			cItem = reinterpret_cast <structPtChunk*>(cList->Items[j]);
			for (k = cItem->Count - 1; k >= 0; k--)
			   {
			   //if (++step < lineStep)
			   if (++_Step < lineStep)
				  continue;
			   PointScreen[ptCount++]  = cItem->Point[k];
			   _Step = 0;
               }
            }
        }
    else
        {
        for (j = 0; j < cList->Count; j++)
            {
            cItem = reinterpret_cast <structPtChunk*>(cList->Items[j]);
            for (k = 0; k < cItem->Count; k++)
               {
               //if (++step < lineStep)
               if (++_Step < lineStep)
                  continue;
			   PointScreen[ptCount++] = cItem->Point[k];
               _Step = 0;
			   }
			}
		}
	}
CountScreen = ptCount;
}
//---------------------------------------------------------------------------
void __fastcall TGrapEllipse::Rotate(TPoint *origin, double angle)
{
TObjectList *cList;
structPtChunk *cItem;
int i, j;
for (i = 0; i < 4; i++)
	{
	cList = chunkList[i];
	for (j = 0; j < cList->Count; j++)
		{
		cItem = reinterpret_cast <structPtChunk*>(cList->Items[j]);
		rotatePoints(cItem->Point, cItem->Count, origin, angle);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TGrapEllipse::Reset(void)
{
for (int i = 0; i < 4; i++)
    clearChunks(chunkList[i]);
}


